package org.gcube.gcat.workspace;

import java.util.List;
import java.util.Map;

import org.gcube.common.homelibrary.home.Home;
import org.gcube.common.homelibrary.home.HomeLibrary;
import org.gcube.common.homelibrary.home.HomeManager;
import org.gcube.common.homelibrary.home.HomeManagerFactory;
import org.gcube.common.homelibrary.home.User;
import org.gcube.common.homelibrary.home.workspace.Workspace;
import org.gcube.common.homelibrary.home.workspace.WorkspaceFolder;
import org.gcube.common.homelibrary.home.workspace.WorkspaceItem;
import org.gcube.common.storagehub.client.dsl.FileContainer;
import org.gcube.common.storagehub.client.dsl.FolderContainer;
import org.gcube.common.storagehub.client.dsl.ItemContainer;
import org.gcube.common.storagehub.client.dsl.ListResolver;
import org.gcube.common.storagehub.model.Metadata;
import org.gcube.common.storagehub.model.items.Item;
import org.gcube.common.storagehub.model.service.Version;
import org.gcube.gcat.ContextTest;
import org.gcube.gcat.utils.ApplicationMode;
import org.gcube.gcat.utils.ContextUtility;
import org.gcube.gcat.workspace.StorageHubManagement;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class StorageHubManagementTest extends ContextTest {
	
	private static final Logger logger = LoggerFactory.getLogger(StorageHubManagementTest.class);
	
	@Test
	public void testHL() throws Exception {
		ApplicationMode applicationMode = new ApplicationMode();
		applicationMode.start();
		String username = ContextUtility.getUsername();
		HomeManagerFactory factory = HomeLibrary.getHomeManagerFactory();
		HomeManager manager = factory.getHomeManager();
		User user = manager.createUser(username);
		@SuppressWarnings("deprecation")
		Home home = manager.getHome(user);
		Workspace ws = home.getWorkspace();
		WorkspaceFolder workspaceFolder = ws.getRoot();
		List<WorkspaceItem> workspaceItems = workspaceFolder.getChildren(true);
		for(WorkspaceItem workspaceItem : workspaceItems) {
			logger.debug("{} {}{}", workspaceFolder.getType(), workspaceItem.getName(), workspaceItem.isHidden()? " (hidden)":"");
		}
	}
	
	@Test
	public void myTest() throws Exception {
		String folderName = StorageHubManagement.getFolderName();
		logger.debug(folderName);
	}
	
	@Test
	public void test() throws Exception {
		ApplicationMode applicationMode = new ApplicationMode();
		applicationMode.start();
		StorageHubManagement storageHubManagement = new StorageHubManagement();
		FolderContainer root = storageHubManagement.getWorkspaceRoot();
		storageHubManagement.tree(root);
	}
	
	@Test
	public void listFolders() throws Exception {
		ApplicationMode applicationMode = new ApplicationMode();
		applicationMode.start();
		StorageHubManagement storageHubManagement = new StorageHubManagement();
		FolderContainer root = storageHubManagement.getWorkspaceRoot();
		storageHubManagement.getCatalogueFolder();
		storageHubManagement.tree(root);
		applicationMode.end();
	}
	
	// @Test
	public void deleteFile() throws Exception {
		StorageHubManagement storageHubManagement = new StorageHubManagement();
		String id = "";
		storageHubManagement.deleteFileByID(id);
	}
	
	@Test
	public void getFileInfo() throws Exception {
		StorageHubManagement storageHubManagement = new StorageHubManagement();
		String id = "";
		FileContainer fileContainer = storageHubManagement.getFileByID(id);
		logger.debug("StorageHub ID {} - File Name {}", id, fileContainer.get().getName());
		ListResolver listResolver = fileContainer.getAnchestors();
		List<ItemContainer<? extends Item>> itemContainers = listResolver.getContainers();
		for(ItemContainer<? extends Item> itemContainer : itemContainers) {
			logger.debug("{}", itemContainer.get().getName());
		}
		
		Metadata metadata = fileContainer.get().getPropertyMap();
		Map<String,Object> map = metadata.getValues();
		logger.debug("{}", map);
		
		List<Version> versions = fileContainer.getVersions();
		for(Version version : versions){
			logger.debug("Version {} {}", version.getId(), version.getName());
		}
	}
	
}
